# -------------------------------------------------------------------------------------
# MKDoc::Functions
# -------------------------------------------------------------------------------------
#
#    Author   : Jean-Michel Hiver                                  (c) MKDoc ltd, 2001.
#
#    Provides utility functions which can be used anywhere in the program.
#
# -------------------------------------------------------------------------------------
package MKDoc::Functions;
use MKDoc::Unicode;
use MKDoc::XML::Dumper;
use Exporter;
use strict;
use 5.6.1;
use utf8;
use CGI::Carp;

our @ISA    = qw /Exporter/;
our @EXPORT = qw /
    local_file_path local_language_ressource local_config local_base_dir
    /;


##
# local_file_path ($file);
# ------------------------
#   Returns the absolute path from the relative path depending
#   on which site is currently being processed
##
sub local_file_path
{
    my $file = shift or confess "\$file is not defined";
    my $base_dir = local_base_dir();
    return $base_dir . '/' . $file;
}


##
# local_base_dir();
# -----------------
#   Returns the absolute base directory for the site which is
#   currently being processed
##
sub local_base_dir
{
    my $server_name = $ENV{SERVER_NAME};
    return '.' unless (defined $server_name);
    
    my $base_dir = $::{P_MKD_NFO}->{$server_name} or
	confess "$server_name has no defined base directory";
    
    $base_dir =~ s/\/$//;
    return $base_dir;
}


##
# local_language_ressource ($lang);
# ---------------------------------
#  need to find out what that's used for... (sigh)
##
sub local_language_ressource
{
    my $dir  = shift;
    my $lang = shift || 'en';
    my @lang = split /-/, $lang;
    
    my $base_dir = local_base_dir();
    
    # tries to find the most appropriate ressource
    while (@lang)
    {
	my $lang = join '-', @lang;
	my $file = $base_dir . '/' . $dir . '/' . $lang . '.xml';
	if (-e $file)
	{
	    open FP, "<$file" or die "Cannot read-open $dir/$lang.xml";
	    my $data = join '', <FP>;
	    close FP;
	    u_mark (\$data);
	    return xml2pl ($data);
	}
	shift (@lang);
    }
    
    # otherwise default to english
    my $file = $base_dir . '/' . $dir . '/en.xml';
    open FP, "<$file" or die "Cannot read-open $dir/en.xml";;
    my $data = join '', <FP>;
    close FP;
    u_mark (\$data);
    
    my $res;
    eval
    {
	$res = xml2pl ($data);
    };
    if (defined $@ and $@)
    {
	die "Cannot dump $dir/$lang.xml\n$@";
    }
    
    return $res;
}


##
# local_config ($classname);
# --------------------------
#   Returns the local config file for $classname. The
#   configuration files are computed as follows:
#
#   This::Is::A::Class  (Class name) <=>
#   This.Is.A.Class.xml (Config file)
##
sub local_config
{
    my $class = shift;
    $class =~ s/::/./g;
    $class .= '.xml';
    
    my $base_dir = local_base_dir();
    my $file = $base_dir . '/' . $class;
    open FP, "<$file" or
	confess "Cannot read-open $file";
    my $data = join '', <FP>;
    close FP;
    
    u_mark (\$data);
    return xml2pl ($data);
}


1;
