# -------------------------------------------------------------------------------------
# MKDoc::Lang
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      Factory class that returns proper language objects
#
# -------------------------------------------------------------------------------------
package MKDoc::Lang;
use MKDoc::Unicode;
use MKDoc::Lang::Generic;
use MKDoc::Lang::Latin1;
use strict;
use 5.6.0;
use utf8;

$::{P_MKD_Lang_Objects} = {};


##
# $class->lang ($lang);
# ---------------------
#   Instanciates the right $lang object depending on $lang
#
#   Presently this method returns the same object
#   if $lang was previously used, but this may be subject
#   to change
#
#   Returns : A language object depending on $lang
##
sub create
{
    my $class = shift;
    my $lang  = u_shift (\@_);
    
    # if the language is covered by the Latin1 charset, then
    # they're quite easy to transliterate => MKDoc::Lang::Latin1
    return new MKDoc::Lang::Latin1
	if ($lang =~ /^(fr|es|ca|eu|pt|it|sq|rm|nl|de|da|sv|no|fi|fo|is|ga|gd|en|af|sw)/i);
   
    # otherwise, let's try to find an appropriate module
    # for the language
    my $module = $lang;
    $module = uc ($module);
    $module =~ tr/-/_/;
    $module = 'MKDoc::Lang::' . $module;
    
    my $result = $class->_get_object ($module);
    return $result if (defined $result);
    
    $module =~ s/_.*?$//;
    $result = $class->_get_object ($module);
    return $result if (defined $result);
    
    # as a last resort, let's use a MKDoc::Lang::Generic
    # language module
    return new MKDoc::Lang::Generic;
}


# utility class method for $class->create;
sub _get_object
{
    my $class = shift;
    my $module = u_shift (\@_);
    
    # attempt to load the module if that was not tempted before
    unless (defined $::{P_MKD_IMP}->{$module})
    {
	eval "use $module";
	$::{P_MKD_IMP}->{$module} = (defined $@ and $@) ? 0 : 1;
    }
    
    # if the module was loaded, then returns an instance of it
    my $object = undef;
    if ($::{P_MKD_IMP}->{$module})
    {
	$::{P_MKD_Lang_Objects}->{$module} ||= $module->new;
	$object = $::{P_MKD_Lang_Objects}->{$module};
    }
    
    return $object;
}


1;
