# -------------------------------------------------------------------------------------
# MKDoc::Lang::Generic
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      Language module for potentially ALL languages, attempts to be as generic
#      as possible.
#
# -------------------------------------------------------------------------------------
package MKDoc::Lang::Generic;
use Unicode::String qw /latin1 utf8/;
use MKDoc::Unicode;
use Text::Unidecode;
use strict;
use 5.6.0;
use utf8;


##
# $self->new;
# -----------
#   Basic constructor for MKDoc::Lang::EN
#
#   @returns - A MKDoc::Lang::EN object
##
sub new
{
    my $class = shift;
    $class = ref $class || $class;
    my $self = bless { @_ }, $class;
    return $self;
}


##
# $self->to_latin ($string);
# --------------------------
#   Returns $string stripped down to latin1 charset
#
#   @param - $string : a utf8 string
#   @returns - A iso latin1 string
##
sub to_latin
{
    my $self   = shift;
    my $string = u_shift (\@_);
    return unidecode ($string);
}


##
# $self->to_ascii ($string);
# --------------------------
#   Returns $string stripped down to ASCII charset
#
#   @param - $string : a utf8 string
#   @returns - A iso ASCII string
##
sub to_ascii
{
    my $self   = shift;
    my $string = u_shift (\@_);
    return unidecode ($string);
}


##
# $self->ascii_keywords ($string);
# --------------------------------
#   Returns a list of ASCII keywords mainly used for indexing
#   from $string
##
sub ascii_keywords
{
    my $self = shift;
    my $value = shift;
    $value = $self->to_ascii ($value);
    
    # uppercase every char
    $value = uc $value;
    
    # turn all wierd char into nothing (accents, commas,...)
    $value =~ s/[^A-Z ]/ /gs;
    
    # remove trailing s'es
    $value =~ s/S\s+/ /gs;
    $value =~ s/S$/ /gs;
    
    my @value = split /\s+/, $value;
    my %res;
    foreach my $value (@value)
    {
	next if ($value =~ /^\s*$/);
	$res{$value} = 1;
    }
    
    return keys %res;
}


1;
