# -------------------------------------------------------------------------------------
# flo::CGI
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@cisedi.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      A handy subclass to allow overriding a couple of CGI.pm methods.
#
# -------------------------------------------------------------------------------------
package flo::CGI;
use MKDoc::Unicode;
use CGI::Carp;
use CGI;
use strict;
use 5.6.0;
use utf8;

our @ISA = qw /CGI/;
use constant FULL_URI => 1;


##
# $self->new;
# -----------
#   Builds a new flo::CGI object
##
sub new
{
    my $class = shift;
    my $self = $class->SUPER::new (@_);
    return $self;
}


##
# $self->self_url;
# ----------------
#   Returns the URI of the current CGI object
##
sub self_url
{
    my $self = shift;    
    my $url = $self->url ( -relative  => ! FULL_URI,
			   -path_info => 1,
			   -query     => 1 );
    
    # remove the blah:80 like URIs
    $url =~ s/(.*?\:\/\/(?:.*?\@)?)(.*):80(?!\d)(.*)/$1$2$3/
	if ($url =~ /(.*?\:\/\/(?:.*?\@)?)(.*):80(?!\d)(.*)/);
    
    return $url;
}


##
# $self->url;
# -----------
#   There is a bug in apache that makes CGI.pm 2.80 behave
#   incorrectly when doing malformed HTTP requests
#   this is the subroutine from CGI.pm 2.79
##
sub url {
    my($self,@p) = CGI::self_or_default(@_);
    my ($relative,$absolute,$full,$path_info,$query,$base) =
      CGI::rearrange(['RELATIVE','ABSOLUTE','FULL',['PATH','PATH_INFO'],['QUERY','QUERY_STRING'],'BASE'] ,@p);
    my $url;
    $full++ if $base || !($relative || $absolute);
    
    my $path = $self->path_info;
    my $script_name = $self->script_name;
    
    if ($full) {
        my $protocol = $self->protocol();
        $url = "$protocol://";
        my $vh = http('host');
        if ($vh) {
            $url .= $vh;
        } else {
            $url .= server_name();
            my $port = $self->server_port;
            $url .= ":" . $port
                unless (lc($protocol) eq 'http' && $port == 80)
                    || (lc($protocol) eq 'https' && $port == 443);
        }
        return $url if $base;
        $url .= $script_name;
    } elsif ($relative) {
        ($url) = $script_name =~ m!([^/]+)$!;
    } elsif ($absolute) {
        $url = $script_name;
    }
    
    $url .= $path if $path_info and defined $path;
    $url .= "?" . $self->query_string if $query and $self->query_string;
    $url = '' unless defined $url;
    $url =~ s/([^a-zA-Z0-9_.%;&?\/\\:+=~-])/uc sprintf("%%%02x",ord($1))/eg;
    return $url;
}


1;
