# -------------------------------------------------------------------------------------
# flo::HTTP::Header
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@cisedi.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      A class that is (mainly) used by the flo::Template to set HTTP headers
#
# -------------------------------------------------------------------------------------
package flo::HTTP::Header;
use MKDoc::Unicode;
use flo::Standard;
use strict;
use utf8;
use 5.6.0;

$::{MKD_flo_HTTP_Header_ETERNAL} = undef;
sub _ETERNAL_()      { return $::{MKD_flo_HTTP_Header_ETERNAL}  }
sub _SET_ETERNAL_($) { $::{MKD_flo_HTTP_Header_ETERNAL} = shift }


##
# $class->new;
# ------------
#   Constructs a new flo::HTTP::Header. This should be done once
#   per query, other bits of code accessing the object via
#   flo::HTTP::Header::Accessor
##
sub new
{
    return _ETERNAL_ if (defined _ETERNAL_);
    my $class = shift;
    $class = ref $class || $class;
    _SET_ETERNAL_ (bless { @_ }, $class);
    return _ETERNAL_;
}


##
# $self->set ($name, $value);
# ---------------------------
#   i.e. $self->set ('Content-Type', 'text/plain');
##
sub set
{
    my $self = shift;
    my $line = u_shift (\@_);
    my ($key, $val) = $line =~ /^\s*(.*?)\s*:\s*(.*?)\s*$/;
    if (defined $key and defined $val) { $self->{$key} = $val }
}


##
# $self->header;
# --------------
#   Returns the HTTP headers string representing that flo::HTTP::Header object
##
sub header
{
    my $self = shift;
    return (join "\n", map { $_ . ": " . $self->{$_} } sort keys %{$self}) . "\n\n";
}


##
# $self->data ($tmpl);
# --------------------
#   Deprecated - Returns HTTP headers + data of the parsed template
##
sub data
{
    my $class  = shift;
    my $tmpl   = u_shift (\@_);
    my $header = new $class;
    my $data   = $tmpl->parse;
    
    no utf8;
    use bytes;
    $header->set ("Content-Length: " . length ($data));
    no bytes;
    use utf8;
    
    return $header->header . $data;
}


##
# $self->data ($out);
# -------------------
#   Deprecated - STDIN prints HTTP headers + data of the parsed template
##
sub out
{
    my $class  = shift;
    my $tmpl   = u_shift (\@_);
    print $class->data ($tmpl);
}


1;
