# -------------------------------------------------------------------------------------
# flo::plugin::CSSFile
# -------------------------------------------------------------------------------------
#
#       Author : Chris Croome (chris@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
#
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      Adapted from Anthony DeLorenzo's <ajdelore@sfu.ca> CssFile script
#      http://www.sfu.ca/~ajdelore/
#
# -------------------------------------------------------------------------------------
package flo::plugin::CSSFile;
use strict;
use lib::sql::Table;
use lib::cgi::CGI;
use flo::Standard;
use lib::Exception;
use lib::cgi::CGI;
use flo::Object;
use flo::CGI;
use flo::HTTP::Header;
use MKDoc::Util::Text2HTML;
use vars qw /@ISA/;
@ISA = qw /flo::Object/;


sub main
{
    my $class = shift;
    my $cgi = lib::cgi::CGI->get;
    
    # if this module is not concerned, let's return ASAP
    my $config = $class->config;
    my $http_name = $config->{http_name};
    my $q_http_name = quotemeta ($http_name);
    my $raw_path_info = raw_path_info;
    return unless ( ($raw_path_info =~ /.*\/$q_http_name$/o) );
    
    # otherwise, let's process the request
    $raw_path_info =~ s/$q_http_name$//;
    my $path_info = $raw_path_info;
    
    # makes all uri's absolute
    $flo::CGI::FULL_URI = 1;

    $class->process();
    return 'TERMINATE';
}


sub process
{
    my $class = shift;
    my $cgi = cgi();
    
    my $set_values = $cgi->param ('setstyle');
    if (defined $set_values and $set_values) { $class->process_set_values }
    else { $class->process_unset_values }
}


sub process_set_values
{
    my $class = shift;
    my $set_values = shift;
    my $cgi = lib::cgi::CGI->get;
    my $config = $class->config;
    my $cookie_expiration = $config->{'cookie_expiration'};
    
    # set the cookie and produce a page to get back to whence they came.
    my $cookie = $cgi->cookie ( -name    => 'stylesheet',
				-value   => $set_values,
				-expires => $cookie_expiration );
    
    my $headers = new flo::HTTP::Header;
    $headers->set ("Set-Cookie: $cookie");
    
    my $document = $class->my_document;
  flo::HTTP::Header->out
      ( new flo::Template ( file => template ('cssfile', $document->{Lang}),
			    hash => \%ENV ) );
}


sub process_unset_values
{
    my $class = shift;
    my $cgi = lib::cgi::CGI->get;
    my $cgi_cookie = $cgi->cookie ('stylesheet');
    my $config = $class->config;
    my $default_stylesheet = $config->{'default_stylesheet'};
    my $cookie_expiration = $config->{'cookie_expiration'};
    
    # if the cookie is set, return the appropriate stylesheets
    if (defined $cgi_cookie and $cgi_cookie)
    {
	my @stylesheets = split /\+/, $cgi->cookie ('stylesheet');
	print $cgi->header (-type => "text/css");
	$class->print_stylesheet ($_) for (@stylesheets);
    }
    
    # otherwise, set a cookie for the default and then send it
    else
    {
	my $cookie = cookie ( -name    => 'stylesheet',
			      -value   => $default_stylesheet,
			      -expires => $cookie_expiration );
	
	print $cgi->header ( -type   => "text/css",
			     -cookie => $cookie );
	
	$class->print_stylesheet ($default_stylesheet);
    }
}


sub print_stylesheet
{
    my $class = shift;
    my $config = $class->config;
    my $css_path = $config->{css_path};
    my $file = shift; 
    
    # strip possible nasties like '../../'
    if ( /(\w+)$/ ) {
	if ( open ( STYLESHEET, "$css_path/$file.css" ) ) {
	    print while <STYLESHEET>;
	}
    }
}


sub my_document
{
    my $class = shift;
    my $config = $class->config;
    my $http_name = $config->{http_name};
    my $q_http_name = quotemeta ($http_name);
    my $raw_path_info = raw_path_info;
    return unless ( ($raw_path_info =~ /.*\/$q_http_name$/o) );
    
    # otherwise, let's process the request
    $raw_path_info =~ s/$q_http_name$//;
    
    # grabs the current document we're working on. If the document doesn't
    # exist, let's return (which should trigger the flo::plugin::Error module)
    my $document_table = table ('Document');
    my $document = $document_table->get ( Full_Path => $raw_path_info ) or return;
    $document_table->lang ($document->{Lang});
    return $document;
}


1;












