#!/usr/bin/perl
use strict;
use diagnostics;

use Getopt::Long;
use CPAN;

use constant MKDOC => qw (
  Config::General
  DBI
  DBD::mysql
  MIME::Base64
  Text::Unicode
  Unicode::String
  Unicode::Normalize
  Unicode::Transliterate
  XML::Parser
  XML::RSS
);

sub docpan
{
  my %option = (upgrade => 0);
  GetOptions(\%option, 'upgrade|u');

  for my $mod (expand("Module", MKDOC))
  {
    next if $mod->uptodate;

    if ($mod->inst_file)
    {
      printf "\nModule %s is installed as %s, could be updated to %s from CPAN\n",
        $mod->id,
        $mod->inst_version,
        $mod->cpan_version;
    }
    else
    {
      printf "\nModule %s is NOT installed. Version %s is available from CPAN\n",
        $mod->id,
        $mod->cpan_version;
    }

    if ($option{upgrade})
    {
      printf "Install/Upgrade %s [y|N] ?", $mod->id;
      my $in = <STDIN>;

      chomp($in);
      $mod->install if (lc($in) eq 'y');
    }
  }
}

docpan();

